"""

   A simple XML-RPC server to expose LoadLeveler data to 
   Non-LoadLeveler machines

   Point a web browser at the server port to see the 
   available function information and data

   Code based on examples in Python Networking Programming

"""

from DocXMLRPCServer import DocXMLRPCServer, DocXMLRPCRequestHandler
#from SimpleXMLRPCServer import SimpleXMLRPCServer, SimpleXMLRPCRequestHandler
from SocketServer import ThreadingMixIn
from time import gmtime, strftime, sleep
from string import join
from pyloadl import *

stepstate = [
    "I", "P", "ST", "R",
    "CP", "XP", "RP",
    "VP", "C", "X", "RM",
    "V", "CA", "NR", "TX",
    "?", "SX", "H", "D",
    "NQ", "E", "EP"
    ]

class ThreadedServer(ThreadingMixIn,DocXMLRPCServer):
        pass

class LoadL:

    def jobs(self):

       """
          Returns LoadLeveler Job Information -

            Returns a multi-dimensional list, with the following data for each dimension:

             element 0 - JobID
             element 1 - Owner
             element 2 - Status
             element 3 - Submit Date
             element 4 - Job Type
             element 5 - Procs
             element 6 - Nodes
             element 7 - Priority
             element 8 - Class
             element 9 - First Machine
             
       """

       data = []
       query = ll_query(JOBS)
       if not PyCObjValid(query):
         ll_deallocate(query)
         return data

       retval = ll_set_request(query, QUERY_ALL, "", ALL_DATA)
       job, number, error = ll_get_objs(query, LL_CM, "")

       if number > 0:

         while PyCObjValid(job):

            cred  = ll_get_data(job, LL_JobCredential)
            owner = ll_get_data(cred, LL_CredentialUserName)

            date = ""
            sdate = ll_get_data(job, LL_JobSubmitTime)
            jname = ll_get_data(job, LL_JobName)
            if sdate != 0:
              sdate = gmtime(sdate)
              date  = strftime("%m/%d %H:%M", sdate)

            step = ll_get_data(job, LL_JobGetFirstStep)
            while PyCObjValid(step):

               state  = ll_get_data(step, LL_StepState)
               status = stepstate[state]

               id     = ll_get_data(step, LL_StepID)
               pri    = ll_get_data(step, LL_StepPriority)
               jclass = ll_get_data(step, LL_StepJobClass)
               nodes  = ll_get_data(step, LL_StepNodeCount)
               type   = ll_get_data(step, LL_StepParallelMode)
               tasks  = ll_get_data(step, LL_StepTaskInstanceCount)

               machine = ""
               machine_obj = ll_get_data(step, LL_StepGetFirstMachine)
               if PyCObjValid(machine_obj):
                  machine = ll_get_data(machine_obj, LL_MachineName)

               data.append([id, jname, owner, status, date, type, nodes, tasks, pri, jclass, machine])

               step = ll_get_data(job, LL_JobGetNextStep)

            job = ll_next_obj(query)

         ll_free_objs(query)
         ll_deallocate(query)

         return data 

    def sleep2(self):
       sleep(30)
       return 1

    def classes(self):

       """
          LoadLeveler Class Information -

            Returns a multi-dimensional list, with the following data for each dimension:

             element 0 - Name
             element 1 - Comment
             element 2 - Priority
             element 3 - Include Users
             element 4 - Maximum Slots
             element 5 - Free Slots
             element 6 - Maximum Processors
             element 7 - WallClock Hard Limit
             element 8 - WallClock Soft Limit

       """

       class_data = []
       query = ll_query(CLASSES)
       if not PyCObjValid(query):
         ll_deallocate(query)
         return class_data

       b = ll_set_request(query, QUERY_ALL, [""], ALL_DATA)

       classes, number, error = ll_get_objs(query, LL_CM, "")

       if number > 0:

         while PyCObjValid(classes):

            class_name       = ll_get_data(classes, LL_ClassName)
            class_comment    = ll_get_data(classes, LL_ClassComment)
            class_prio       = ll_get_data(classes, LL_ClassPriority)
            class_inc_users  = ll_get_data(classes, LL_ClassIncludeUsers)
            class_max_procs  = ll_get_data(classes, LL_ClassMaxProcessors)
            class_free_slots = ll_get_data(classes, LL_ClassFreeSlots)
            class_max_slots  = ll_get_data(classes, LL_ClassMaximumSlots)
            class_wcl_hlimit = ll_get_data(classes, LL_ClassWallClockLimitHard)
            class_wcl_slimit = ll_get_data(classes, LL_ClassWallClockLimitSoft)
            class_inc_users  = join(class_inc_users, ',')

            class_data.append([class_name, class_comment, class_prio, class_inc_users, class_max_slots, class_free_slots, class_max_procs, class_wcl_hlimit, class_wcl_slimit])

            classes = ll_next_obj(query)

       ll_free_objs(query)
       ll_deallocate(query)

       return class_data

if __name__ == "__main__":

   #host = "192.168.0.5"
   host = ""
   port = 7895
   serveraddr = (host, port)

   loadl_object = LoadL()
   server = ThreadedServer(serveraddr, DocXMLRPCRequestHandler)
   server.set_server_title("LoadLeveler Remote Access Server")
   server.set_server_documentation("Welcome to the LoadLeveler DocXMLRPCServer")
   server.register_instance(loadl_object)
   server.register_multicall_functions()

   print "Listening on %s port %d" % (host, port)
   server.serve_forever()
